using System;
using System.Collections.Generic;
using System.Text;
using System.Data;

using CMS.IECommerce;
using CMS.CMSEcommerce;
using CMS.GlobalHelper;
using CMS.SettingsProvider;
using CMS.SiteProvider;

namespace CMS.CustomECommerceProvider
{
    /// <summary>
    /// Custom shipping option info provider
    /// </summary>
    public class CustomShippingOptionInfoProvider : IShippingOptionInfoProvider
    {
        /// <summary>
        /// Returns the ShippingOptionInfo structure for the specified shippingOption.
        /// </summary>
        /// <param name="shippingOptionId">ShippingOption id.</param>
        public object GetShippingOptionInfo(int shippingOptionId)
        {
            return ShippingOptionInfoProvider.GetShippingOptionInfo(shippingOptionId);
        }


        /// <summary>
        /// Returns the ShippingOptionInfo structure for the specified shippingOption.
        /// </summary>
        /// <param name="shippingOptionName">ShippingOptionName.</param>
        /// <param name="siteName">Site name.</param>
        public object GetShippingOptionInfo(string shippingOptionName, string siteName)
        {
            return ShippingOptionInfoProvider.GetShippingOptionInfo(shippingOptionName, siteName);
        }


        /// <summary>
        /// Sets (updates or inserts) specified shippingOption.
        /// </summary>
        /// <param name="shippingOption">ShippingOption to set.</param>
        public void SetShippingOptionInfo(object shippingOption)
        {
            ShippingOptionInfoProvider.SetShippingOptionInfo((CMS.Ecommerce.ShippingOptionInfo)shippingOption);
        }


        /// <summary>
        /// Deletes specified shippingOption.
        /// </summary>
        /// <param name="shippingOption">ShippingOption object.</param>
        public void DeleteShippingOptionInfo(object shippingOption)
        {
            ShippingOptionInfoProvider.DeleteShippingOptionInfo((CMS.Ecommerce.ShippingOptionInfo)shippingOption);
        }


        /// <summary>
        /// Deletes specified shippingOption.
        /// </summary>
        /// <param name="shippingOptionId">ShippingOption id.</param>
        public void DeleteShippingOptionInfo(int shippingOptionId)
        {
            ShippingOptionInfoProvider.DeleteShippingOptionInfo(shippingOptionId);
        }


        /// <summary>
        /// Check dependencies, return true if is depend
        /// </summary>
        public bool CheckDependencies(int shippingOptionID)
        {
            return ShippingOptionInfoProvider.CheckDependencies(shippingOptionID);
        }


        /// <summary>
        /// Gets all Shipping options.
        /// </summary>
        public DataSet GetShippingOptions()
        {
            return ShippingOptionInfoProvider.GetShippingOptions();
        }


        /// <summary>
        /// Gets Shipping options.
        /// </summary>
        /// <param name="where">Where condition</param>
        /// <param name="orderBy">Order by clause</param>
        public DataSet GetShippingOptions(string where, string orderBy)
        {
            return ShippingOptionInfoProvider.GetShippingOptions(where, orderBy);
        }


        /// <summary>
        /// Returns DataSet with all shipping options. 
        /// </summary> 
        /// <param name="where">Where condition to filter data</param> 
        /// <param name="orderBy">Order by statement</param> 
        /// <param name="topN">Specifies number of returned records</param> 
        /// <param name="columns">Data columns to return</param> 
        public DataSet GetShippingOptions(string where, string orderBy, int topN, string columns)
        {
            return ShippingOptionInfoProvider.GetShippingOptions(where, orderBy, topN, columns);
        }


        /// <summary>
        /// Get shippings options.
        /// </summary>
        /// <param name="cart">Shopping cart.</param>
        /// <param name="status">Indicates shipping options that should be returned.</param>
        public DataSet GetShippingOptions(object cart, object status)
        {
            return ShippingOptionInfoProvider.GetAllShippingsForSite(((CMS.Ecommerce.ShoppingCartInfo)cart).ShoppingCartSiteID, (CMS.Ecommerce.StatusEnum)status);
        }


        /// <summary>
        /// Get all shippings for specific site.
        /// </summary>
        /// <param name="siteId">Site ID.</param>
        /// <param name="status">Indicates shipping options that should be returned.</param>
        public DataSet GetAllShippingsForSite(int siteId, object status)
        {
            return ShippingOptionInfoProvider.GetAllShippingsForSite(siteId, (CMS.Ecommerce.StatusEnum)status);
        }


        /// <summary>
        /// Returns the shipping cost for given shopping cart.
        /// </summary>
        /// <param name="cartObj">Shopping cart object.</param>
        /// <param name="siteName">Site name.</param>
        public double CalculateShipping(object cartObj, string siteName)
        {

           CMS.Ecommerce.ShoppingCartInfo cart = cartObj as CMS.Ecommerce.ShoppingCartInfo;

           if (cart != null)
           {
               // Get shipping address details
               CMS.Ecommerce.AddressInfo address = CMS.Ecommerce.AddressInfoProvider.GetAddressInfo(cart.ShoppingCartShippingAddressID);
               if (address != null)
               {
                   // Get shipping address country and state
                   CountryInfo country = CountryInfoProvider.GetCountryInfo(address.AddressCountryID);
                   StateInfo state = StateInfoProvider.GetStateInfo(address.AddressStateID);
               
                   // Add tax to standard shipping price for countries/states with specified "ShippingTax"
                   CMS.Ecommerce.TaxClassInfo shippingTaxClassInfo = TaxClassInfoProvider.GetTaxClassInfo("ShippingTax");
                   
                   if (shippingTaxClassInfo != null)
                   {
                       // tax will be zero if no value found for state/country
                       double shippingTax = 0;
                       if (state != null)
                       {
                           CMS.Ecommerce.TaxClassStateInfo tcsi = CMS.Ecommerce.TaxClassStateInfoProvider.GetTaxClassStateInfo(shippingTaxClassInfo.TaxClassID, state.StateID);
                           if (tcsi != null)
                           {
                               shippingTax = tcsi.TaxValue;
                           }
                       }
                       else
                       {
                           CMS.Ecommerce.TaxClassCountryInfo tcci = CMS.Ecommerce.TaxClassCountryInfoProvider.GetTaxClassCountryInfo(country.CountryID, shippingTaxClassInfo.TaxClassID);
                           if (tcci != null)
                           {
                               shippingTax = tcci.TaxValue;
                           }
                       }

                       // Perform calculation of shipping charge for current shopping cart
                       double shippingWithoutTax = CMS.CMSEcommerce.ShippingOptionInfoProvider.CalculateShipping(cart, siteName);

                       // Calculate tax amount
                       double calculatedShippingTax = TaxClassInfoProvider.GetTaxValue(shippingWithoutTax, shippingTax, false);

                       // Store calculated tax amount into shop.cart custom data
                       cart.ShoppingCartCustomData["ShippingTax"] = calculatedShippingTax.ToString();
                       cart.UpdateShoppingCartCustomData();

                       // Return shipping charge including calculated tax
                       return shippingWithoutTax + calculatedShippingTax;
                   }
               }
           }
           // Get standard shipping price in other cases
           return CMS.CMSEcommerce.ShippingOptionInfoProvider.CalculateShipping(cart, siteName);
        }


        /// <summary>
        /// Return shipping price or zero according to site store shipping free limit.
        /// </summary>
        /// <param name="shipping">Shipping price.</param>
        /// <param name="totalPrice">Total order price.</param>
        /// <param name="siteStoreFreeLimit">Site store shipping free limit.</param>
        public double ApplyShippingFreeLimit(double shipping, double totalPrice, double siteStoreFreeLimit)
        {
            return ShippingOptionInfoProvider.ApplyShippingFreeLimit(shipping, totalPrice, siteStoreFreeLimit);
        }
    }
}
